from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="AccountPlan")


@_attrs_define
class AccountPlan:
    """
    Attributes:
        max_user_count (Union[Unset, int]):
        name (Union[Unset, str]):
        is_yearly (Union[Unset, bool]):
    """

    max_user_count: Union[Unset, int] = UNSET
    name: Union[Unset, str] = UNSET
    is_yearly: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        max_user_count = self.max_user_count

        name = self.name

        is_yearly = self.is_yearly

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if max_user_count is not UNSET:
            field_dict["maxUserCount"] = max_user_count
        if name is not UNSET:
            field_dict["name"] = name
        if is_yearly is not UNSET:
            field_dict["isYearly"] = is_yearly

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        max_user_count = d.pop("maxUserCount", UNSET)

        name = d.pop("name", UNSET)

        is_yearly = d.pop("isYearly", UNSET)

        account_plan = cls(
            max_user_count=max_user_count,
            name=name,
            is_yearly=is_yearly,
        )

        account_plan.additional_properties = d
        return account_plan

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import argparse
import os

from desktop_notifier import DesktopNotifier
from textual import on
from textual.app import App
from textual.binding import Binding

import tygenie.config as config
import tygenie.logger as logger
import tygenie.opsgenie as opsgenie
import tygenie.screens.add_note
import tygenie.screens.alerts
import tygenie.screens.settings
from tygenie import consts
from tygenie.alert_details.description_formatter import ContentFormatter
from tygenie.alerts_list.formatter import AlertFormatter


class TygenieApp(App):
    """Opsgenie terminal application"""

    CSS_PATH = f'{os.path.join(os.path.dirname(__file__),"tygenie.tcss")}'
    BINDINGS = [
        Binding(
            key="ctrl+s",
            action=f'switch_screen("{consts.SETTINGS_SCREEN_NAME}")',
            description="Settings",
        ),
        Binding(key="ctrl+q", action="quit", description="Quit"),
    ]

    SCREENS = {
        consts.ALERTS_SCREEN_NAME: tygenie.screens.alerts.AlertsScreen,
        consts.SETTINGS_SCREEN_NAME: tygenie.screens.settings.SettingsScreen,
        consts.ADD_NOTE_SCREEN_NAME: tygenie.screens.add_note.AddNoteScreen,
    }

    COMMAND_PALETTE_BINDING = config.ty_config.tygenie.get("keybindings", {}).get(
        "palette", "ctrl+p"
    )

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.notifier = DesktopNotifier(
            app_icon=f"{os.path.dirname(__file__)}/assets/icon.png"
        )
        self.started = False
        self.load_plugins()

    def load_plugins(self):
        plugins = config.ty_config.tygenie.get("plugins", {})
        self.alerts_list_formatter = AlertFormatter(
            formatter=plugins.get("alerts_list_formatter", None), app=self.app
        )
        self.alert_description_formatter = ContentFormatter(
            formatter=plugins.get("alert_description_formatter", None)
        )

    @on(tygenie.screens.settings.SettingsScreen.SettingsUpdated)
    async def reload(self):
        self.load_plugins()
        self.refresh_bindings()
        await self.get_screen(consts.ALERTS_SCREEN_NAME).reload()

    def get_theme_variable_defaults(self) -> dict[str, str]:
        theme_config = self.get_theme(self.theme)
        if theme_config is None:
            return {}

        return {
            "open": theme_config.error or "",
            "acked": theme_config.accent or "",
            "closed": theme_config.success or "",
        }

    async def on_mount(self):
        if config.ty_config.sample_copied:
            self.push_screen(consts.SETTINGS_SCREEN_NAME)
        else:
            self.push_screen(consts.ALERTS_SCREEN_NAME)
        dn_config = config.ty_config.tygenie.get("desktop_notification", {})
        if dn_config.get("enable", True) is True:
            await self.notifier.send(
                title="Tygenie",
                message=f"Starting tygenie {consts.VERSION}",
            )
        self.theme = config.ty_config.tygenie.get("theme", "flexoki")


def main() -> None:
    parser = argparse.ArgumentParser()
    parser.add_argument("--config", "-c", default=None)
    arguments = parser.parse_args()

    if arguments.config is not None:
        config.set_config_file(arguments.config)
        logger.reload()
        opsgenie.reload()

    app = TygenieApp()
    try:
        app.run()
    except Exception:
        pass


if __name__ == "__main__":
    main()



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.user_meta import UserMeta


T = TypeVar("T", bound="ForwardingRule")


@_attrs_define
class ForwardingRule:
    """
    Attributes:
        from_user (Union[Unset, UserMeta]):
        to_user (Union[Unset, UserMeta]):
        start_date (Union[Unset, datetime.datetime]):
        end_date (Union[Unset, datetime.datetime]):
        alias (Union[Unset, str]):
        id (Union[Unset, str]):
    """

    from_user: Union[Unset, "UserMeta"] = UNSET
    to_user: Union[Unset, "UserMeta"] = UNSET
    start_date: Union[Unset, datetime.datetime] = UNSET
    end_date: Union[Unset, datetime.datetime] = UNSET
    alias: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        from_user: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.from_user, Unset):
            from_user = self.from_user.to_dict()

        to_user: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.to_user, Unset):
            to_user = self.to_user.to_dict()

        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        alias = self.alias

        id = self.id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if from_user is not UNSET:
            field_dict["fromUser"] = from_user
        if to_user is not UNSET:
            field_dict["toUser"] = to_user
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if alias is not UNSET:
            field_dict["alias"] = alias
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.user_meta import UserMeta

        d = src_dict.copy()
        _from_user = d.pop("fromUser", UNSET)
        from_user: Union[Unset, UserMeta]
        if isinstance(_from_user, Unset):
            from_user = UNSET
        else:
            from_user = UserMeta.from_dict(_from_user)

        _to_user = d.pop("toUser", UNSET)
        to_user: Union[Unset, UserMeta]
        if isinstance(_to_user, Unset):
            to_user = UNSET
        else:
            to_user = UserMeta.from_dict(_to_user)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.datetime]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date)

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, datetime.datetime]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date)

        alias = d.pop("alias", UNSET)

        id = d.pop("id", UNSET)

        forwarding_rule = cls(
            from_user=from_user,
            to_user=to_user,
            start_date=start_date,
            end_date=end_date,
            alias=alias,
            id=id,
        )

        forwarding_rule.additional_properties = d
        return forwarding_rule

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DeduplicationActionDeduplicationActionType(str, Enum):
    FREQUENCY_BASED = "frequency-based"
    VALUE_BASED = "value-based"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.account_plan import AccountPlan


T = TypeVar("T", bound="AccountInfo")


@_attrs_define
class AccountInfo:
    """
    Attributes:
        name (Union[Unset, str]):
        user_count (Union[Unset, int]):
        plan (Union[Unset, AccountPlan]):
    """

    name: Union[Unset, str] = UNSET
    user_count: Union[Unset, int] = UNSET
    plan: Union[Unset, "AccountPlan"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        user_count = self.user_count

        plan: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.plan, Unset):
            plan = self.plan.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if user_count is not UNSET:
            field_dict["userCount"] = user_count
        if plan is not UNSET:
            field_dict["plan"] = plan

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.account_plan import AccountPlan

        d = src_dict.copy()
        name = d.pop("name", UNSET)

        user_count = d.pop("userCount", UNSET)

        _plan = d.pop("plan", UNSET)
        plan: Union[Unset, AccountPlan]
        if isinstance(_plan, Unset):
            plan = UNSET
        else:
            plan = AccountPlan.from_dict(_plan)

        account_info = cls(
            name=name,
            user_count=user_count,
            plan=plan,
        )

        account_info.additional_properties = d
        return account_info

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import pendulum
from textual import log

import tygenie.config as config


class Logger:

    def __init__(self):
        self._load_config()

    def _load_config(self):
        self.enable = config.ty_config.tygenie.get("log", {}).get("enable", False)
        self.file = config.ty_config.tygenie.get("log", {}).get(
            "file", "/tmp/tygenie.log"
        )

    def log(self, message: str = ""):
        if not message or not self.enable:
            return

        date = pendulum.now()
        logline = f"[{date}] {message}"
        # Logline visble in textual console: textual console -vvv
        # and run app.py file with textual run app.py --dev
        log(f"{logline}")
        try:
            with open(self.file, "a") as f:
                f.write(logline + "\n")
        except Exception as e:
            log(f"Unable to log in file: {e}")
            pass


logger = Logger()


def reload():
    global logger
    logger = Logger()



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.update_service_payload_visibility import UpdateServicePayloadVisibility
from ..types import UNSET, Unset

T = TypeVar("T", bound="UpdateServicePayload")


@_attrs_define
class UpdateServicePayload:
    """
    Attributes:
        name (str): Name of the service
        description (Union[Unset, str]): Description field of the service that is generally used to provide a detailed
            information about the service.
        visibility (Union[Unset, UpdateServicePayloadVisibility]):
    """

    name: str
    description: Union[Unset, str] = UNSET
    visibility: Union[Unset, UpdateServicePayloadVisibility] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        description = self.description

        visibility: Union[Unset, str] = UNSET
        if not isinstance(self.visibility, Unset):
            visibility = self.visibility.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if visibility is not UNSET:
            field_dict["visibility"] = visibility

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        description = d.pop("description", UNSET)

        _visibility = d.pop("visibility", UNSET)
        visibility: Union[Unset, UpdateServicePayloadVisibility]
        if isinstance(_visibility, Unset):
            visibility = UNSET
        else:
            visibility = UpdateServicePayloadVisibility(_visibility)

        update_service_payload = cls(
            name=name,
            description=description,
            visibility=visibility,
        )

        update_service_payload.additional_properties = d
        return update_service_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.base_integration_action import BaseIntegrationAction
    from ..models.common_integration_action import CommonIntegrationAction
    from ..models.create_integration_action import CreateIntegrationAction
    from ..models.integration_meta import IntegrationMeta


T = TypeVar("T", bound="ActionCategorized")


@_attrs_define
class ActionCategorized:
    """
    Attributes:
        field_parent (Union[Unset, IntegrationMeta]):
        ignore (Union[Unset, List['BaseIntegrationAction']]):
        create (Union[Unset, List['CreateIntegrationAction']]):
        close (Union[Unset, List['CommonIntegrationAction']]):
        acknowledge (Union[Unset, List['CommonIntegrationAction']]):
        add_note (Union[Unset, List['CommonIntegrationAction']]):
    """

    field_parent: Union[Unset, "IntegrationMeta"] = UNSET
    ignore: Union[Unset, List["BaseIntegrationAction"]] = UNSET
    create: Union[Unset, List["CreateIntegrationAction"]] = UNSET
    close: Union[Unset, List["CommonIntegrationAction"]] = UNSET
    acknowledge: Union[Unset, List["CommonIntegrationAction"]] = UNSET
    add_note: Union[Unset, List["CommonIntegrationAction"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field_parent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.field_parent, Unset):
            field_parent = self.field_parent.to_dict()

        ignore: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.ignore, Unset):
            ignore = []
            for ignore_item_data in self.ignore:
                ignore_item = ignore_item_data.to_dict()
                ignore.append(ignore_item)

        create: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.create, Unset):
            create = []
            for create_item_data in self.create:
                create_item = create_item_data.to_dict()
                create.append(create_item)

        close: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.close, Unset):
            close = []
            for close_item_data in self.close:
                close_item = close_item_data.to_dict()
                close.append(close_item)

        acknowledge: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.acknowledge, Unset):
            acknowledge = []
            for acknowledge_item_data in self.acknowledge:
                acknowledge_item = acknowledge_item_data.to_dict()
                acknowledge.append(acknowledge_item)

        add_note: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.add_note, Unset):
            add_note = []
            for add_note_item_data in self.add_note:
                add_note_item = add_note_item_data.to_dict()
                add_note.append(add_note_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if field_parent is not UNSET:
            field_dict["_parent"] = field_parent
        if ignore is not UNSET:
            field_dict["ignore"] = ignore
        if create is not UNSET:
            field_dict["create"] = create
        if close is not UNSET:
            field_dict["close"] = close
        if acknowledge is not UNSET:
            field_dict["acknowledge"] = acknowledge
        if add_note is not UNSET:
            field_dict["addNote"] = add_note

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.base_integration_action import BaseIntegrationAction
        from ..models.common_integration_action import CommonIntegrationAction
        from ..models.create_integration_action import CreateIntegrationAction
        from ..models.integration_meta import IntegrationMeta

        d = src_dict.copy()
        _field_parent = d.pop("_parent", UNSET)
        field_parent: Union[Unset, IntegrationMeta]
        if isinstance(_field_parent, Unset):
            field_parent = UNSET
        else:
            field_parent = IntegrationMeta.from_dict(_field_parent)

        ignore = []
        _ignore = d.pop("ignore", UNSET)
        for ignore_item_data in _ignore or []:
            ignore_item = BaseIntegrationAction.from_dict(ignore_item_data)

            ignore.append(ignore_item)

        create = []
        _create = d.pop("create", UNSET)
        for create_item_data in _create or []:
            create_item = CreateIntegrationAction.from_dict(create_item_data)

            create.append(create_item)

        close = []
        _close = d.pop("close", UNSET)
        for close_item_data in _close or []:
            close_item = CommonIntegrationAction.from_dict(close_item_data)

            close.append(close_item)

        acknowledge = []
        _acknowledge = d.pop("acknowledge", UNSET)
        for acknowledge_item_data in _acknowledge or []:
            acknowledge_item = CommonIntegrationAction.from_dict(acknowledge_item_data)

            acknowledge.append(acknowledge_item)

        add_note = []
        _add_note = d.pop("addNote", UNSET)
        for add_note_item_data in _add_note or []:
            add_note_item = CommonIntegrationAction.from_dict(add_note_item_data)

            add_note.append(add_note_item)

        action_categorized = cls(
            field_parent=field_parent,
            ignore=ignore,
            create=create,
            close=close,
            acknowledge=acknowledge,
            add_note=add_note,
        )

        action_categorized.additional_properties = d
        return action_categorized

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.contact_meta import ContactMeta
    from ..models.duration import Duration
    from ..models.notification_rule_step_parent import NotificationRuleStepParent


T = TypeVar("T", bound="NotificationRuleStep")


@_attrs_define
class NotificationRuleStep:
    """
    Attributes:
        field_parent (Union[Unset, NotificationRuleStepParent]):
        id (Union[Unset, str]):
        send_after (Union[Unset, Duration]):
        contact (Union[Unset, ContactMeta]):
        enabled (Union[Unset, bool]):
    """

    field_parent: Union[Unset, "NotificationRuleStepParent"] = UNSET
    id: Union[Unset, str] = UNSET
    send_after: Union[Unset, "Duration"] = UNSET
    contact: Union[Unset, "ContactMeta"] = UNSET
    enabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field_parent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.field_parent, Unset):
            field_parent = self.field_parent.to_dict()

        id = self.id

        send_after: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.send_after, Unset):
            send_after = self.send_after.to_dict()

        contact: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.contact, Unset):
            contact = self.contact.to_dict()

        enabled = self.enabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if field_parent is not UNSET:
            field_dict["_parent"] = field_parent
        if id is not UNSET:
            field_dict["id"] = id
        if send_after is not UNSET:
            field_dict["sendAfter"] = send_after
        if contact is not UNSET:
            field_dict["contact"] = contact
        if enabled is not UNSET:
            field_dict["enabled"] = enabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.contact_meta import ContactMeta
        from ..models.duration import Duration
        from ..models.notification_rule_step_parent import NotificationRuleStepParent

        d = src_dict.copy()
        _field_parent = d.pop("_parent", UNSET)
        field_parent: Union[Unset, NotificationRuleStepParent]
        if isinstance(_field_parent, Unset):
            field_parent = UNSET
        else:
            field_parent = NotificationRuleStepParent.from_dict(_field_parent)

        id = d.pop("id", UNSET)

        _send_after = d.pop("sendAfter", UNSET)
        send_after: Union[Unset, Duration]
        if isinstance(_send_after, Unset):
            send_after = UNSET
        else:
            send_after = Duration.from_dict(_send_after)

        _contact = d.pop("contact", UNSET)
        contact: Union[Unset, ContactMeta]
        if isinstance(_contact, Unset):
            contact = UNSET
        else:
            contact = ContactMeta.from_dict(_contact)

        enabled = d.pop("enabled", UNSET)

        notification_rule_step = cls(
            field_parent=field_parent,
            id=id,
            send_after=send_after,
            contact=contact,
            enabled=enabled,
        )

        notification_rule_step.additional_properties = d
        return notification_rule_step

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.saved_search_meta import SavedSearchMeta


T = TypeVar("T", bound="ListSavedSearchesResponse")


@_attrs_define
class ListSavedSearchesResponse:
    """
    Attributes:
        request_id (str):
        took (float):  Default: 0.0.
        data (Union[Unset, List['SavedSearchMeta']]):
    """

    request_id: str
    took: float = 0.0
    data: Union[Unset, List["SavedSearchMeta"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        request_id = self.request_id

        took = self.took

        data: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.data, Unset):
            data = []
            for data_item_data in self.data:
                data_item = data_item_data.to_dict()
                data.append(data_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "requestId": request_id,
                "took": took,
            }
        )
        if data is not UNSET:
            field_dict["data"] = data

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.saved_search_meta import SavedSearchMeta

        d = src_dict.copy()
        request_id = d.pop("requestId")

        took = d.pop("took")

        data = []
        _data = d.pop("data", UNSET)
        for data_item_data in _data or []:
            data_item = SavedSearchMeta.from_dict(data_item_data)

            data.append(data_item)

        list_saved_searches_response = cls(
            request_id=request_id,
            took=took,
            data=data,
        )

        list_saved_searches_response.additional_properties = d
        return list_saved_searches_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ServiceIncidentConditionOperation(str, Enum):
    CONTAINS = "contains"
    CONTAINS_KEY = "contains-key"
    CONTAINS_VALUE = "contains-value"
    ENDS_WITH = "ends-with"
    EQUALS = "equals"
    EQUALS_IGNORE_WHITESPACE = "equals-ignore-whitespace"
    GREATER_THAN = "greater-than"
    IS_EMPTY = "is-empty"
    LESS_THAN = "less-than"
    MATCHES = "matches"
    STARTS_WITH = "starts-with"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.alert_user_meta import AlertUserMeta


T = TypeVar("T", bound="AlertRecipient")


@_attrs_define
class AlertRecipient:
    """
    Attributes:
        user (Union[Unset, AlertUserMeta]):
        state (Union[Unset, str]):
        method (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
    """

    user: Union[Unset, "AlertUserMeta"] = UNSET
    state: Union[Unset, str] = UNSET
    method: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.user, Unset):
            user = self.user.to_dict()

        state = self.state

        method = self.method

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if user is not UNSET:
            field_dict["user"] = user
        if state is not UNSET:
            field_dict["state"] = state
        if method is not UNSET:
            field_dict["method"] = method
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.alert_user_meta import AlertUserMeta

        d = src_dict.copy()
        _user = d.pop("user", UNSET)
        user: Union[Unset, AlertUserMeta]
        if isinstance(_user, Unset):
            user = UNSET
        else:
            user = AlertUserMeta.from_dict(_user)

        state = d.pop("state", UNSET)

        method = d.pop("method", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        alert_recipient = cls(
            user=user,
            state=state,
            method=method,
            created_at=created_at,
            updated_at=updated_at,
        )

        alert_recipient.additional_properties = d
        return alert_recipient

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class BidirectionalCallbackBidirectionalCallbackType(str, Enum):
    CONNECT_WISE_CALLBACK = "connect-wise-callback"
    DESK_CALLBACK = "desk-callback"
    ES_WATCHER_CALLBACK = "es-watcher-callback"
    HIP_CHAT_ADD_ON_CALLBACK = "hip-chat-add-on-callback"
    HIP_CHAT_CALLBACK_V2 = "hip-chat-callback-v2"
    ICINGA2_CALLBACK = "icinga2-callback"
    ICINGA_CALLBACK = "icinga-callback"
    MARID_CALLBACK = "marid-callback"
    MATTERMOST_CALLBACK = "mattermost-callback"
    NAGIOS_BASED_V1_CALLBACK = "nagios-based-v1-callback"
    NAGIOS_BASED_V2_CALLBACK = "nagios-based-v2-callback"
    NAGIOS_XIV1_CALLBACK = "nagios-xiv1-callback"
    NAGIOS_XIV2_CALLBACK = "nagios-xiv2-callback"
    SLACK_APP_CALLBACK = "slack-app-callback"
    SLACK_CALLBACK = "slack-callback"
    SOLARWINDS_CALLBACK = "solarwinds-callback"
    SOLAR_WINDS_WEB_HELP_DESK_CALLBACK = "solar-winds-web-help-desk-callback"
    STACKDRIVER_CALLBACK = "stackdriver-callback"
    STATUS_IO_CALLBACK = "status-io-callback"
    TRACK_IT_CALLBACK = "track-it-callback"
    XMPP_CALLBACK = "xmpp-callback"
    ZABBIX_CALLBACK = "zabbix-callback"
    ZENOSS_CALLBACK = "zenoss-callback"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.contact_meta import ContactMeta
    from ..models.duration import Duration


T = TypeVar("T", bound="CreateNotificationRuleStepPayload")


@_attrs_define
class CreateNotificationRuleStepPayload:
    """
    Attributes:
        contact (ContactMeta):
        enabled (bool): Specifies whether given step will be enabled or not when it is created.
        send_after (Union[Unset, Duration]):
    """

    contact: "ContactMeta"
    enabled: bool
    send_after: Union[Unset, "Duration"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        contact = self.contact.to_dict()

        enabled = self.enabled

        send_after: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.send_after, Unset):
            send_after = self.send_after.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "contact": contact,
                "enabled": enabled,
            }
        )
        if send_after is not UNSET:
            field_dict["sendAfter"] = send_after

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.contact_meta import ContactMeta
        from ..models.duration import Duration

        d = src_dict.copy()
        contact = ContactMeta.from_dict(d.pop("contact"))

        enabled = d.pop("enabled")

        _send_after = d.pop("sendAfter", UNSET)
        send_after: Union[Unset, Duration]
        if isinstance(_send_after, Unset):
            send_after = UNSET
        else:
            send_after = Duration.from_dict(_send_after)

        create_notification_rule_step_payload = cls(
            contact=contact,
            enabled=enabled,
            send_after=send_after,
        )

        create_notification_rule_step_payload.additional_properties = d
        return create_notification_rule_step_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.contact_status import ContactStatus


T = TypeVar("T", bound="ContactWithApplyOrder")


@_attrs_define
class ContactWithApplyOrder:
    """
    Attributes:
        id (Union[Unset, str]):
        method (Union[Unset, str]):
        to (Union[Unset, str]):
        status (Union[Unset, ContactStatus]):
        apply_order (Union[Unset, int]):
    """

    id: Union[Unset, str] = UNSET
    method: Union[Unset, str] = UNSET
    to: Union[Unset, str] = UNSET
    status: Union[Unset, "ContactStatus"] = UNSET
    apply_order: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        method = self.method

        to = self.to

        status: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.to_dict()

        apply_order = self.apply_order

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if method is not UNSET:
            field_dict["method"] = method
        if to is not UNSET:
            field_dict["to"] = to
        if status is not UNSET:
            field_dict["status"] = status
        if apply_order is not UNSET:
            field_dict["applyOrder"] = apply_order

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.contact_status import ContactStatus

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        method = d.pop("method", UNSET)

        to = d.pop("to", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, ContactStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = ContactStatus.from_dict(_status)

        apply_order = d.pop("applyOrder", UNSET)

        contact_with_apply_order = cls(
            id=id,
            method=method,
            to=to,
            status=status,
            apply_order=apply_order,
        )

        contact_with_apply_order.additional_properties = d
        return contact_with_apply_order

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.alert_details import AlertDetails
    from ..models.alert_integration import AlertIntegration
    from ..models.alert_report import AlertReport
    from ..models.responder import Responder


T = TypeVar("T", bound="Alert")


@_attrs_define
class Alert:
    """
    Attributes:
        id (str):
        tiny_id (Union[Unset, str]):
        alias (Union[Unset, str]):
        message (Union[Unset, str]):
        status (Union[Unset, str]):
        acknowledged (Union[Unset, bool]):
        is_seen (Union[Unset, bool]):
        tags (Union[Unset, List[str]]):
        snoozed (Union[Unset, bool]):
        snoozed_until (Union[Unset, datetime.datetime]):
        count (Union[Unset, int]):
        last_occurred_at (Union[Unset, datetime.datetime]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
        source (Union[Unset, str]):
        owner (Union[Unset, str]):
        priority (Union[Unset, str]):
        responders (Union[Unset, List['Responder']]):
        integration (Union[Unset, AlertIntegration]):
        report (Union[Unset, AlertReport]):
        actions (Union[Unset, List[str]]):
        entity (Union[Unset, str]):
        description (Union[Unset, str]):
        details (Union[Unset, AlertDetails]):
    """

    id: str
    tiny_id: Union[Unset, str] = UNSET
    alias: Union[Unset, str] = UNSET
    message: Union[Unset, str] = UNSET
    status: Union[Unset, str] = UNSET
    acknowledged: Union[Unset, bool] = UNSET
    is_seen: Union[Unset, bool] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    snoozed: Union[Unset, bool] = UNSET
    snoozed_until: Union[Unset, datetime.datetime] = UNSET
    count: Union[Unset, int] = UNSET
    last_occurred_at: Union[Unset, datetime.datetime] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    source: Union[Unset, str] = UNSET
    owner: Union[Unset, str] = UNSET
    priority: Union[Unset, str] = UNSET
    responders: Union[Unset, List["Responder"]] = UNSET
    integration: Union[Unset, "AlertIntegration"] = UNSET
    report: Union[Unset, "AlertReport"] = UNSET
    actions: Union[Unset, List[str]] = UNSET
    entity: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    details: Union[Unset, "AlertDetails"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        tiny_id = self.tiny_id

        alias = self.alias

        message = self.message

        status = self.status

        acknowledged = self.acknowledged

        is_seen = self.is_seen

        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        snoozed = self.snoozed

        snoozed_until: Union[Unset, str] = UNSET
        if not isinstance(self.snoozed_until, Unset):
            snoozed_until = self.snoozed_until.isoformat()

        count = self.count

        last_occurred_at: Union[Unset, str] = UNSET
        if not isinstance(self.last_occurred_at, Unset):
            last_occurred_at = self.last_occurred_at.isoformat()

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        source = self.source

        owner = self.owner

        priority = self.priority

        responders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.responders, Unset):
            responders = []
            for responders_item_data in self.responders:
                responders_item = responders_item_data.to_dict()
                responders.append(responders_item)

        integration: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.integration, Unset):
            integration = self.integration.to_dict()

        report: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.report, Unset):
            report = self.report.to_dict()

        actions: Union[Unset, List[str]] = UNSET
        if not isinstance(self.actions, Unset):
            actions = self.actions

        entity = self.entity

        description = self.description

        details: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.details, Unset):
            details = self.details.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
            }
        )
        if tiny_id is not UNSET:
            field_dict["tinyId"] = tiny_id
        if alias is not UNSET:
            field_dict["alias"] = alias
        if message is not UNSET:
            field_dict["message"] = message
        if status is not UNSET:
            field_dict["status"] = status
        if acknowledged is not UNSET:
            field_dict["acknowledged"] = acknowledged
        if is_seen is not UNSET:
            field_dict["isSeen"] = is_seen
        if tags is not UNSET:
            field_dict["tags"] = tags
        if snoozed is not UNSET:
            field_dict["snoozed"] = snoozed
        if snoozed_until is not UNSET:
            field_dict["snoozedUntil"] = snoozed_until
        if count is not UNSET:
            field_dict["count"] = count
        if last_occurred_at is not UNSET:
            field_dict["lastOccurredAt"] = last_occurred_at
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at
        if source is not UNSET:
            field_dict["source"] = source
        if owner is not UNSET:
            field_dict["owner"] = owner
        if priority is not UNSET:
            field_dict["priority"] = priority
        if responders is not UNSET:
            field_dict["responders"] = responders
        if integration is not UNSET:
            field_dict["integration"] = integration
        if report is not UNSET:
            field_dict["report"] = report
        if actions is not UNSET:
            field_dict["actions"] = actions
        if entity is not UNSET:
            field_dict["entity"] = entity
        if description is not UNSET:
            field_dict["description"] = description
        if details is not UNSET:
            field_dict["details"] = details

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.alert_details import AlertDetails
        from ..models.alert_integration import AlertIntegration
        from ..models.alert_report import AlertReport
        from ..models.responder import Responder

        d = src_dict.copy()
        id = d.pop("id")

        tiny_id = d.pop("tinyId", UNSET)

        alias = d.pop("alias", UNSET)

        message = d.pop("message", UNSET)

        status = d.pop("status", UNSET)

        acknowledged = d.pop("acknowledged", UNSET)

        is_seen = d.pop("isSeen", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        snoozed = d.pop("snoozed", UNSET)

        _snoozed_until = d.pop("snoozedUntil", UNSET)
        snoozed_until: Union[Unset, datetime.datetime]
        if isinstance(_snoozed_until, Unset):
            snoozed_until = UNSET
        else:
            snoozed_until = isoparse(_snoozed_until)

        count = d.pop("count", UNSET)

        _last_occurred_at = d.pop("lastOccurredAt", UNSET)
        last_occurred_at: Union[Unset, datetime.datetime]
        if isinstance(_last_occurred_at, Unset):
            last_occurred_at = UNSET
        else:
            last_occurred_at = isoparse(_last_occurred_at)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        source = d.pop("source", UNSET)

        owner = d.pop("owner", UNSET)

        priority = d.pop("priority", UNSET)

        responders = []
        _responders = d.pop("responders", UNSET)
        for responders_item_data in _responders or []:
            responders_item = Responder.from_dict(responders_item_data)

            responders.append(responders_item)

        _integration = d.pop("integration", UNSET)
        integration: Union[Unset, AlertIntegration]
        if isinstance(_integration, Unset):
            integration = UNSET
        else:
            integration = AlertIntegration.from_dict(_integration)

        _report = d.pop("report", UNSET)
        report: Union[Unset, AlertReport]
        if isinstance(_report, Unset):
            report = UNSET
        else:
            report = AlertReport.from_dict(_report)

        actions = cast(List[str], d.pop("actions", UNSET))

        entity = d.pop("entity", UNSET)

        description = d.pop("description", UNSET)

        _details = d.pop("details", UNSET)
        details: Union[Unset, AlertDetails]
        if isinstance(_details, Unset):
            details = UNSET
        else:
            details = AlertDetails.from_dict(_details)

        alert = cls(
            id=id,
            tiny_id=tiny_id,
            alias=alias,
            message=message,
            status=status,
            acknowledged=acknowledged,
            is_seen=is_seen,
            tags=tags,
            snoozed=snoozed,
            snoozed_until=snoozed_until,
            count=count,
            last_occurred_at=last_occurred_at,
            created_at=created_at,
            updated_at=updated_at,
            source=source,
            owner=owner,
            priority=priority,
            responders=responders,
            integration=integration,
            report=report,
            actions=actions,
            entity=entity,
            description=description,
            details=details,
        )

        alert.additional_properties = d
        return alert

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class EscalationRuleNotifyType(str, Enum):
    ADMINS = "admins"
    ALL = "all"
    DEFAULT = "default"
    NEXT = "next"
    PREVIOUS = "previous"
    USERS = "users"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="HeartbeatMeta")


@_attrs_define
class HeartbeatMeta:
    """
    Attributes:
        name (Union[Unset, str]):
        enabled (Union[Unset, bool]):
        expired (Union[Unset, bool]):
    """

    name: Union[Unset, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    expired: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        enabled = self.enabled

        expired = self.expired

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if expired is not UNSET:
            field_dict["expired"] = expired

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        enabled = d.pop("enabled", UNSET)

        expired = d.pop("expired", UNSET)

        heartbeat_meta = cls(
            name=name,
            enabled=enabled,
            expired=expired,
        )

        heartbeat_meta.additional_properties = d
        return heartbeat_meta

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="TimeOfDayRestriction")


@_attrs_define
class TimeOfDayRestriction:
    """
    Attributes:
        start_hour (Union[Unset, int]):
        start_min (Union[Unset, int]):
        end_hour (Union[Unset, int]):
        end_min (Union[Unset, int]):
    """

    start_hour: Union[Unset, int] = UNSET
    start_min: Union[Unset, int] = UNSET
    end_hour: Union[Unset, int] = UNSET
    end_min: Union[Unset, int] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        start_hour = self.start_hour

        start_min = self.start_min

        end_hour = self.end_hour

        end_min = self.end_min

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if start_hour is not UNSET:
            field_dict["startHour"] = start_hour
        if start_min is not UNSET:
            field_dict["startMin"] = start_min
        if end_hour is not UNSET:
            field_dict["endHour"] = end_hour
        if end_min is not UNSET:
            field_dict["endMin"] = end_min

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        start_hour = d.pop("startHour", UNSET)

        start_min = d.pop("startMin", UNSET)

        end_hour = d.pop("endHour", UNSET)

        end_min = d.pop("endMin", UNSET)

        time_of_day_restriction = cls(
            start_hour=start_hour,
            start_min=start_min,
            end_hour=end_hour,
            end_min=end_min,
        )

        time_of_day_restriction.additional_properties = d
        return time_of_day_restriction

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from textual.app import ComposeResult
from textual.containers import VerticalScroll
from textual.reactive import reactive
from textual.widget import Widget
from textual.widgets import Checkbox
from typing import List


class Checkboxes(Widget):

    names: reactive[List[str]] = reactive([], recompose=True)

    def __init__(self, names: list[str] = [], **kwargs):
        super().__init__(**kwargs)
        if names:
            self.update(names=names)

    def compose(self) -> ComposeResult:
        with VerticalScroll():
            for name in self.names:
                yield Checkbox(name, value=True, name=name)

    def update(self, names: List[str]):
        self.names = names

    def watch_names(self, names: List[str]):
        self.update(names=names)



from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.user_contact_contact_method import UserContactContactMethod
from ..types import UNSET, Unset

T = TypeVar("T", bound="UserContact")


@_attrs_define
class UserContact:
    """
    Attributes:
        to (Union[Unset, str]):
        id (Union[Unset, str]):
        contact_method (Union[Unset, UserContactContactMethod]):
        disabled_reason (Union[Unset, str]):
        enabled (Union[Unset, bool]):
    """

    to: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    contact_method: Union[Unset, UserContactContactMethod] = UNSET
    disabled_reason: Union[Unset, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        to = self.to

        id = self.id

        contact_method: Union[Unset, str] = UNSET
        if not isinstance(self.contact_method, Unset):
            contact_method = self.contact_method.value

        disabled_reason = self.disabled_reason

        enabled = self.enabled

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if to is not UNSET:
            field_dict["to"] = to
        if id is not UNSET:
            field_dict["id"] = id
        if contact_method is not UNSET:
            field_dict["contactMethod"] = contact_method
        if disabled_reason is not UNSET:
            field_dict["disabledReason"] = disabled_reason
        if enabled is not UNSET:
            field_dict["enabled"] = enabled

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        to = d.pop("to", UNSET)

        id = d.pop("id", UNSET)

        _contact_method = d.pop("contactMethod", UNSET)
        contact_method: Union[Unset, UserContactContactMethod]
        if isinstance(_contact_method, Unset):
            contact_method = UNSET
        else:
            contact_method = UserContactContactMethod(_contact_method)

        disabled_reason = d.pop("disabledReason", UNSET)

        enabled = d.pop("enabled", UNSET)

        user_contact = cls(
            to=to,
            id=id,
            contact_method=contact_method,
            disabled_reason=disabled_reason,
            enabled=enabled,
        )

        user_contact.additional_properties = d
        return user_contact

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class OutgoingCallbackNewCallbackType(str, Enum):
    AMAZON_SNS_CALLBACK = "amazon-sns-callback"
    BASE_WEBHOOK_CALLBACK = "base-webhook-callback"
    BIDIRECTIONAL_CALLBACK_NEW = "bidirectional-callback-new"
    BMC_REMEDY_ON_DEMAND_CALLBACK = "bmc-remedy-on-demand-callback"
    OEC_CALLBACK = "oec-callback"

    def __str__(self) -> str:
        return str(self.value)



from textual.containers import VerticalScroll
from textual.message import Message
from textual.widget import Widget
from textual.widgets import ContentSwitcher, Label, Markdown

import tygenie.opsgenie as opsgenie
from tygenie.widgets.center_middle import CenterMiddle


class AlertNotes(Widget):

    DEFAULT_CSS = """\
    AlertNotes {
        & #no_note_label {
            height: 1fr;
            hatch: right $surface-lighten-1 70%;
        }
    }
"""

    def compose(self):
        self.border_title = "Alert notes"
        content_switcher = ContentSwitcher(initial=None, id="alert_notes_switcher")
        with content_switcher:
            yield CenterMiddle(
                Label("The alert doesn't have a note"), id="no_note_label"
            )
            with VerticalScroll(id="alert_notes_markdown_container"):
                md = Markdown(None, name="Notes", id="md_alert_note")
                yield md

    class UpdateAlertNotes(Message):

        def __init__(self, alert_notes) -> None:
            self.alert_notes = alert_notes.data
            super().__init__()

    async def get_alert_notes(self, opsgenie_id: str | None = None):
        if not opsgenie_id:
            return None
        alert_notes = await opsgenie.client.api.get_alert_notes(
            parameters={
                "identifier": opsgenie_id,
                "identifier_type": "id",
            }
        )

        if alert_notes is not None:
            self.post_message(self.UpdateAlertNotes(alert_notes=alert_notes))
        else:
            self.notify(severity="error", message="Unable to get alert note")



from enum import Enum


class MaintenanceMetaStatus(str, Enum):
    ACTIVE = "active"
    CANCELLED = "cancelled"
    PAST = "past"
    PLANNED = "planned"

    def __str__(self) -> str:
        return str(self.value)



import re
from importlib import import_module

from markdownify import markdownify as md
from textual import log


class BaseContentFormatter:

    def __init__(self, content: str = "") -> None:
        self.execution_order = {"pre": [], "post": []}
        self.content = content
        self.methods = []

    def __init_subclass__(cls, **kwargs) -> None:
        super().__init_subclass__(**kwargs)
        cls.methods = sorted(dir(cls))

    def _format(self, step: str = ""):
        if not step:
            return

        for method in self.execution_order[step]:
            if method.startswith(f"{step}_substitute_"):
                result = getattr(self, method)()
                log(
                    f"Substituing method {method}: {result['regexp']} by {result['sub']}"
                )
                self.content = re.sub(result["regexp"], result["sub"], self.content)
                continue

            if method.startswith(f"{step}_custom_"):
                log(f"Running custom method {method}")
                getattr(self, method)()
                continue

    def _pre_format(self):
        self._format(step="pre")

    def _post_format(self):
        self._format(step="post")

    def format(self):
        self._pre_format()
        self.content = md(
            self.content,
            default_title=True,
            escape_misc=False,
            escape_asterisks=False,
            escape_underscores=False,
        )
        self._post_format()
        return self.content


class ContentFormatter:

    def __init__(self, content: str = "", formatter: str | None = None) -> None:
        self.content: str = content
        self.formatter: str | None = formatter
        if formatter is None:
            self.module = import_module("tygenie.alert_details.description_formatter")
            self.formatter = "DefaultContentFormatter"
        else:
            self.module = import_module(
                f"tygenie.plugins.{self.formatter}.alert_description_formatter"
            )

    def format(self):
        self.content = getattr(self.module, self.formatter)(
            content=self.content
        ).format()
        return self.content


class DefaultContentFormatter(BaseContentFormatter):
    """Default content formatter"""



from enum import Enum


class NotificationActionType(str, Enum):
    ACKNOWLEDGED_ALERT = "acknowledged-alert"
    ADD_NOTE = "add-note"
    ASSIGNED_ALERT = "assigned-alert"
    CLOSED_ALERT = "closed-alert"
    CREATE_ALERT = "create-alert"
    INCOMING_CALL_ROUTING = "incoming-call-routing"
    RENOTIFIED_ALERT = "renotified-alert"
    SCHEDULE_END = "schedule-end"
    SCHEDULE_START = "schedule-start"

    def __str__(self) -> str:
        return str(self.value)



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="SnoozeAlertPayload")


@_attrs_define
class SnoozeAlertPayload:
    """
    Attributes:
        end_time (datetime.datetime): Date and time that snooze will lose effect
        user (Union[Unset, str]): Display name of the request owner
        note (Union[Unset, str]): Additional note that will be added while creating the alert
        source (Union[Unset, str]): Source field of the alert. Default value is IP address of the incoming request
    """

    end_time: datetime.datetime
    user: Union[Unset, str] = UNSET
    note: Union[Unset, str] = UNSET
    source: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        end_time = self.end_time.isoformat()

        user = self.user

        note = self.note

        source = self.source

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "endTime": end_time,
            }
        )
        if user is not UNSET:
            field_dict["user"] = user
        if note is not UNSET:
            field_dict["note"] = note
        if source is not UNSET:
            field_dict["source"] = source

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        end_time = isoparse(d.pop("endTime"))

        user = d.pop("user", UNSET)

        note = d.pop("note", UNSET)

        source = d.pop("source", UNSET)

        snooze_alert_payload = cls(
            end_time=end_time,
            user=user,
            note=note,
            source=source,
        )

        snooze_alert_payload.additional_properties = d
        return snooze_alert_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.team_recipient import TeamRecipient
    from ..models.user_recipient import UserRecipient


T = TypeVar("T", bound="UpdateSavedSearchPayload")


@_attrs_define
class UpdateSavedSearchPayload:
    """
    Attributes:
        name (str):
        query (str):
        owner (UserRecipient): User recipient
        description (Union[Unset, str]):
        teams (Union[Unset, List['TeamRecipient']]): Teams that the alert will be routed to send notifications
    """

    name: str
    query: str
    owner: "UserRecipient"
    description: Union[Unset, str] = UNSET
    teams: Union[Unset, List["TeamRecipient"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        query = self.query

        owner = self.owner.to_dict()

        description = self.description

        teams: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.teams, Unset):
            teams = []
            for teams_item_data in self.teams:
                teams_item = teams_item_data.to_dict()
                teams.append(teams_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "query": query,
                "owner": owner,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if teams is not UNSET:
            field_dict["teams"] = teams

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.team_recipient import TeamRecipient
        from ..models.user_recipient import UserRecipient

        d = src_dict.copy()
        name = d.pop("name")

        query = d.pop("query")

        owner = UserRecipient.from_dict(d.pop("owner"))

        description = d.pop("description", UNSET)

        teams = []
        _teams = d.pop("teams", UNSET)
        for teams_item_data in _teams or []:
            teams_item = TeamRecipient.from_dict(teams_item_data)

            teams.append(teams_item)

        update_saved_search_payload = cls(
            name=name,
            query=query,
            owner=owner,
            description=description,
            teams=teams,
        )

        update_saved_search_payload.additional_properties = d
        return update_saved_search_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.recipient import Recipient
    from ..models.schedule_meta import ScheduleMeta
    from ..models.schedule_override_rotation import ScheduleOverrideRotation


T = TypeVar("T", bound="ScheduleOverride")


@_attrs_define
class ScheduleOverride:
    """
    Attributes:
        field_parent (Union[Unset, ScheduleMeta]):
        alias (Union[Unset, str]):
        user (Union[Unset, Recipient]):
        start_date (Union[Unset, datetime.datetime]):
        end_date (Union[Unset, datetime.datetime]):
        rotations (Union[Unset, List['ScheduleOverrideRotation']]):
    """

    field_parent: Union[Unset, "ScheduleMeta"] = UNSET
    alias: Union[Unset, str] = UNSET
    user: Union[Unset, "Recipient"] = UNSET
    start_date: Union[Unset, datetime.datetime] = UNSET
    end_date: Union[Unset, datetime.datetime] = UNSET
    rotations: Union[Unset, List["ScheduleOverrideRotation"]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field_parent: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.field_parent, Unset):
            field_parent = self.field_parent.to_dict()

        alias = self.alias

        user: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.user, Unset):
            user = self.user.to_dict()

        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        rotations: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.rotations, Unset):
            rotations = []
            for rotations_item_data in self.rotations:
                rotations_item = rotations_item_data.to_dict()
                rotations.append(rotations_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if field_parent is not UNSET:
            field_dict["_parent"] = field_parent
        if alias is not UNSET:
            field_dict["alias"] = alias
        if user is not UNSET:
            field_dict["user"] = user
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if rotations is not UNSET:
            field_dict["rotations"] = rotations

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.recipient import Recipient
        from ..models.schedule_meta import ScheduleMeta
        from ..models.schedule_override_rotation import ScheduleOverrideRotation

        d = src_dict.copy()
        _field_parent = d.pop("_parent", UNSET)
        field_parent: Union[Unset, ScheduleMeta]
        if isinstance(_field_parent, Unset):
            field_parent = UNSET
        else:
            field_parent = ScheduleMeta.from_dict(_field_parent)

        alias = d.pop("alias", UNSET)

        _user = d.pop("user", UNSET)
        user: Union[Unset, Recipient]
        if isinstance(_user, Unset):
            user = UNSET
        else:
            user = Recipient.from_dict(_user)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.datetime]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date)

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, datetime.datetime]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date)

        rotations = []
        _rotations = d.pop("rotations", UNSET)
        for rotations_item_data in _rotations or []:
            rotations_item = ScheduleOverrideRotation.from_dict(rotations_item_data)

            rotations.append(rotations_item)

        schedule_override = cls(
            field_parent=field_parent,
            alias=alias,
            user=user,
            start_date=start_date,
            end_date=end_date,
            rotations=rotations,
        )

        schedule_override.additional_properties = d
        return schedule_override

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from textual.app import ComposeResult
from textual.containers import HorizontalGroup
from textual.message import Message
from textual.widget import Widget
from textual.widgets import Input, Static

import tygenie.config as config


class TagValueInput(Widget):

    def compose(self) -> ComposeResult:
        default_tag = config.ty_config.tygenie.get("default_tag", "")
        tag_label: Static = Static(
            name="tag_label", id="tag_label", content="Tag value: "
        )
        tag_input: Input = Input(name="tag_input", id="tag_input", value=default_tag)
        yield HorizontalGroup(tag_label, tag_input, id="tag_value_horizontal_group")

    class TagValueChange(Message):
        """A message to indicate that we tag value changed"""

        def __init__(self, label: str | None = None):
            self.label = label
            super().__init__()

    def on_input_submitted(self, submit: Input.Submitted):
        if submit.input.id == "tag_input":
            self.notify(
                severity="information", message=f"tag is now set to '{submit.value}'"
            )
            tag = self.query_one("#tag_input", Input)
            self.post_message(self.TagValueChange(label=tag.value))
            self.app.query_one("#alerts_data_table").focus(True)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.filter_ import Filter
    from ..models.recipient import Recipient
    from ..models.time_restriction_interval import TimeRestrictionInterval


T = TypeVar("T", bound="TeamRoutingRule")


@_attrs_define
class TeamRoutingRule:
    """
    Attributes:
        id (Union[Unset, str]):
        name (Union[Unset, str]):
        is_default (Union[Unset, bool]):
        order (Union[Unset, int]):
        criteria (Union[Unset, Filter]): Defines the conditions that will be checked before applying rules and type of
            the operations that will be applied on conditions
        timezone (Union[Unset, str]):
        time_restriction (Union[Unset, TimeRestrictionInterval]):
        notify (Union[Unset, Recipient]):
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    is_default: Union[Unset, bool] = UNSET
    order: Union[Unset, int] = UNSET
    criteria: Union[Unset, "Filter"] = UNSET
    timezone: Union[Unset, str] = UNSET
    time_restriction: Union[Unset, "TimeRestrictionInterval"] = UNSET
    notify: Union[Unset, "Recipient"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name

        is_default = self.is_default

        order = self.order

        criteria: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.criteria, Unset):
            criteria = self.criteria.to_dict()

        timezone = self.timezone

        time_restriction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.time_restriction, Unset):
            time_restriction = self.time_restriction.to_dict()

        notify: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.notify, Unset):
            notify = self.notify.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if is_default is not UNSET:
            field_dict["isDefault"] = is_default
        if order is not UNSET:
            field_dict["order"] = order
        if criteria is not UNSET:
            field_dict["criteria"] = criteria
        if timezone is not UNSET:
            field_dict["timezone"] = timezone
        if time_restriction is not UNSET:
            field_dict["timeRestriction"] = time_restriction
        if notify is not UNSET:
            field_dict["notify"] = notify

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.filter_ import Filter
        from ..models.recipient import Recipient
        from ..models.time_restriction_interval import TimeRestrictionInterval

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        is_default = d.pop("isDefault", UNSET)

        order = d.pop("order", UNSET)

        _criteria = d.pop("criteria", UNSET)
        criteria: Union[Unset, Filter]
        if isinstance(_criteria, Unset):
            criteria = UNSET
        else:
            criteria = Filter.from_dict(_criteria)

        timezone = d.pop("timezone", UNSET)

        _time_restriction = d.pop("timeRestriction", UNSET)
        time_restriction: Union[Unset, TimeRestrictionInterval]
        if isinstance(_time_restriction, Unset):
            time_restriction = UNSET
        else:
            time_restriction = TimeRestrictionInterval.from_dict(_time_restriction)

        _notify = d.pop("notify", UNSET)
        notify: Union[Unset, Recipient]
        if isinstance(_notify, Unset):
            notify = UNSET
        else:
            notify = Recipient.from_dict(_notify)

        team_routing_rule = cls(
            id=id,
            name=name,
            is_default=is_default,
            order=order,
            criteria=criteria,
            timezone=timezone,
            time_restriction=time_restriction,
            notify=notify,
        )

        team_routing_rule.additional_properties = d
        return team_routing_rule

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.create_schedule_rotation_payload_type import CreateScheduleRotationPayloadType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.recipient import Recipient
    from ..models.time_restriction_interval import TimeRestrictionInterval


T = TypeVar("T", bound="CreateScheduleRotationPayload")


@_attrs_define
class CreateScheduleRotationPayload:
    """
    Attributes:
        start_date (datetime.datetime): Defines a date time as an override start. Minutes may take 0 or 30 as value.
            Otherwise they will be converted to nearest 0 or 30 automatically
        type (CreateScheduleRotationPayloadType): Type of rotation. May be one of 'daily', 'weekly' and 'hourly'
        participants (List['Recipient']): List of escalations, teams, users or the reserved word none which will be used
            in schedule. Each of them can be used multiple times and will be rotated in the order they given.
        name (Union[Unset, str]): Name of rotation
        end_date (Union[Unset, datetime.datetime]): Defines a date time as an override end. Minutes may take 0 or 30 as
            value. Otherwise they will be converted to nearest 0 or 30 automatically
        length (Union[Unset, int]): Length of the rotation with default value 1
        time_restriction (Union[Unset, TimeRestrictionInterval]):
    """

    start_date: datetime.datetime
    type: CreateScheduleRotationPayloadType
    participants: List["Recipient"]
    name: Union[Unset, str] = UNSET
    end_date: Union[Unset, datetime.datetime] = UNSET
    length: Union[Unset, int] = UNSET
    time_restriction: Union[Unset, "TimeRestrictionInterval"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        start_date = self.start_date.isoformat()

        type = self.type.value

        participants = []
        for participants_item_data in self.participants:
            participants_item = participants_item_data.to_dict()
            participants.append(participants_item)

        name = self.name

        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        length = self.length

        time_restriction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.time_restriction, Unset):
            time_restriction = self.time_restriction.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "startDate": start_date,
                "type": type,
                "participants": participants,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if length is not UNSET:
            field_dict["length"] = length
        if time_restriction is not UNSET:
            field_dict["timeRestriction"] = time_restriction

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.recipient import Recipient
        from ..models.time_restriction_interval import TimeRestrictionInterval

        d = src_dict.copy()
        start_date = isoparse(d.pop("startDate"))

        type = CreateScheduleRotationPayloadType(d.pop("type"))

        participants = []
        _participants = d.pop("participants")
        for participants_item_data in _participants:
            participants_item = Recipient.from_dict(participants_item_data)

            participants.append(participants_item)

        name = d.pop("name", UNSET)

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, datetime.datetime]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date)

        length = d.pop("length", UNSET)

        _time_restriction = d.pop("timeRestriction", UNSET)
        time_restriction: Union[Unset, TimeRestrictionInterval]
        if isinstance(_time_restriction, Unset):
            time_restriction = UNSET
        else:
            time_restriction = TimeRestrictionInterval.from_dict(_time_restriction)

        create_schedule_rotation_payload = cls(
            start_date=start_date,
            type=type,
            participants=participants,
            name=name,
            end_date=end_date,
            length=length,
            time_restriction=time_restriction,
        )

        create_schedule_rotation_payload.additional_properties = d
        return create_schedule_rotation_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ServiceIncidentPropertiesPriority(str, Enum):
    P1 = "P1"
    P2 = "P2"
    P3 = "P3"
    P4 = "P4"
    P5 = "P5"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class ServiceIncidentConditionField(str, Enum):
    DESCRIPTION = "description"
    EXTRA_PROPERTIES = "extra-properties"
    MESSAGE = "message"
    PRIORITY = "priority"
    RECIPIENTS = "recipients"
    TAGS = "tags"
    TEAMS = "teams"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class TimeRestrictionIntervalType(str, Enum):
    TIME_OF_DAY = "time-of-day"
    WEEKDAY_AND_TIME_OF_DAY = "weekday-and-time-of-day"

    def __str__(self) -> str:
        return str(self.value)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.integration_type import IntegrationType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.team_meta import TeamMeta


T = TypeVar("T", bound="Integration")


@_attrs_define
class Integration:
    """
    Attributes:
        type (IntegrationType): Type of the integration. (For instance, "API" for API Integration)
        name (str): Name of the integration. Name must be unique for each integration
        id (Union[Unset, str]):
        enabled (Union[Unset, bool]): This parameter is for specifying whether the integration will be enabled or not
        owner_team (Union[Unset, TeamMeta]):
        is_global (Union[Unset, bool]):
        field_read_only (Union[Unset, List[str]]):
    """

    type: IntegrationType
    name: str
    id: Union[Unset, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    owner_team: Union[Unset, "TeamMeta"] = UNSET
    is_global: Union[Unset, bool] = UNSET
    field_read_only: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        name = self.name

        id = self.id

        enabled = self.enabled

        owner_team: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner_team, Unset):
            owner_team = self.owner_team.to_dict()

        is_global = self.is_global

        field_read_only: Union[Unset, List[str]] = UNSET
        if not isinstance(self.field_read_only, Unset):
            field_read_only = self.field_read_only

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "name": name,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if owner_team is not UNSET:
            field_dict["ownerTeam"] = owner_team
        if is_global is not UNSET:
            field_dict["isGlobal"] = is_global
        if field_read_only is not UNSET:
            field_dict["_readOnly"] = field_read_only

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.team_meta import TeamMeta

        d = src_dict.copy()
        type = IntegrationType(d.pop("type"))

        name = d.pop("name")

        id = d.pop("id", UNSET)

        enabled = d.pop("enabled", UNSET)

        _owner_team = d.pop("ownerTeam", UNSET)
        owner_team: Union[Unset, TeamMeta]
        if isinstance(_owner_team, Unset):
            owner_team = UNSET
        else:
            owner_team = TeamMeta.from_dict(_owner_team)

        is_global = d.pop("isGlobal", UNSET)

        field_read_only = cast(List[str], d.pop("_readOnly", UNSET))

        integration = cls(
            type=type,
            name=name,
            id=id,
            enabled=enabled,
            owner_team=owner_team,
            is_global=is_global,
            field_read_only=field_read_only,
        )

        integration.additional_properties = d
        return integration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.saved_search_entity import SavedSearchEntity


T = TypeVar("T", bound="SavedSearch")


@_attrs_define
class SavedSearch:
    """
    Attributes:
        id (Union[Unset, str]):
        name (Union[Unset, str]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
        owner (Union[Unset, SavedSearchEntity]):
        teams (Union[Unset, List['SavedSearchEntity']]):
        description (Union[Unset, str]):
        query (Union[Unset, str]):
    """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    owner: Union[Unset, "SavedSearchEntity"] = UNSET
    teams: Union[Unset, List["SavedSearchEntity"]] = UNSET
    description: Union[Unset, str] = UNSET
    query: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        owner: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner, Unset):
            owner = self.owner.to_dict()

        teams: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.teams, Unset):
            teams = []
            for teams_item_data in self.teams:
                teams_item = teams_item_data.to_dict()
                teams.append(teams_item)

        description = self.description

        query = self.query

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at
        if owner is not UNSET:
            field_dict["owner"] = owner
        if teams is not UNSET:
            field_dict["teams"] = teams
        if description is not UNSET:
            field_dict["description"] = description
        if query is not UNSET:
            field_dict["query"] = query

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.saved_search_entity import SavedSearchEntity

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        _owner = d.pop("owner", UNSET)
        owner: Union[Unset, SavedSearchEntity]
        if isinstance(_owner, Unset):
            owner = UNSET
        else:
            owner = SavedSearchEntity.from_dict(_owner)

        teams = []
        _teams = d.pop("teams", UNSET)
        for teams_item_data in _teams or []:
            teams_item = SavedSearchEntity.from_dict(teams_item_data)

            teams.append(teams_item)

        description = d.pop("description", UNSET)

        query = d.pop("query", UNSET)

        saved_search = cls(
            id=id,
            name=name,
            created_at=created_at,
            updated_at=updated_at,
            owner=owner,
            teams=teams,
            description=description,
            query=query,
        )

        saved_search.additional_properties = d
        return saved_search

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.update_schedule_rotation_payload_type import UpdateScheduleRotationPayloadType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.recipient import Recipient
    from ..models.time_restriction_interval import TimeRestrictionInterval


T = TypeVar("T", bound="UpdateScheduleRotationPayload")


@_attrs_define
class UpdateScheduleRotationPayload:
    """
    Attributes:
        name (Union[Unset, str]): Name of rotation
        start_date (Union[Unset, datetime.datetime]): Defines a date time as an override start. Minutes may take 0 or 30
            as value. Otherwise they will be converted to nearest 0 or 30 automatically
        end_date (Union[Unset, datetime.datetime]): Defines a date time as an override end. Minutes may take 0 or 30 as
            value. Otherwise they will be converted to nearest 0 or 30 automatically
        type (Union[Unset, UpdateScheduleRotationPayloadType]): Type of rotation. May be one of 'daily', 'weekly' and
            'hourly'
        length (Union[Unset, int]): Length of the rotation with default value 1
        participants (Union[Unset, List['Recipient']]): List of escalations, teams, users or the reserved word none
            which will be used in schedule. Each of them can be used multiple times and will be rotated in the order they
            given.
        time_restriction (Union[Unset, TimeRestrictionInterval]):
    """

    name: Union[Unset, str] = UNSET
    start_date: Union[Unset, datetime.datetime] = UNSET
    end_date: Union[Unset, datetime.datetime] = UNSET
    type: Union[Unset, UpdateScheduleRotationPayloadType] = UNSET
    length: Union[Unset, int] = UNSET
    participants: Union[Unset, List["Recipient"]] = UNSET
    time_restriction: Union[Unset, "TimeRestrictionInterval"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        start_date: Union[Unset, str] = UNSET
        if not isinstance(self.start_date, Unset):
            start_date = self.start_date.isoformat()

        end_date: Union[Unset, str] = UNSET
        if not isinstance(self.end_date, Unset):
            end_date = self.end_date.isoformat()

        type: Union[Unset, str] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type.value

        length = self.length

        participants: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.participants, Unset):
            participants = []
            for participants_item_data in self.participants:
                participants_item = participants_item_data.to_dict()
                participants.append(participants_item)

        time_restriction: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.time_restriction, Unset):
            time_restriction = self.time_restriction.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if start_date is not UNSET:
            field_dict["startDate"] = start_date
        if end_date is not UNSET:
            field_dict["endDate"] = end_date
        if type is not UNSET:
            field_dict["type"] = type
        if length is not UNSET:
            field_dict["length"] = length
        if participants is not UNSET:
            field_dict["participants"] = participants
        if time_restriction is not UNSET:
            field_dict["timeRestriction"] = time_restriction

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.recipient import Recipient
        from ..models.time_restriction_interval import TimeRestrictionInterval

        d = src_dict.copy()
        name = d.pop("name", UNSET)

        _start_date = d.pop("startDate", UNSET)
        start_date: Union[Unset, datetime.datetime]
        if isinstance(_start_date, Unset):
            start_date = UNSET
        else:
            start_date = isoparse(_start_date)

        _end_date = d.pop("endDate", UNSET)
        end_date: Union[Unset, datetime.datetime]
        if isinstance(_end_date, Unset):
            end_date = UNSET
        else:
            end_date = isoparse(_end_date)

        _type = d.pop("type", UNSET)
        type: Union[Unset, UpdateScheduleRotationPayloadType]
        if isinstance(_type, Unset):
            type = UNSET
        else:
            type = UpdateScheduleRotationPayloadType(_type)

        length = d.pop("length", UNSET)

        participants = []
        _participants = d.pop("participants", UNSET)
        for participants_item_data in _participants or []:
            participants_item = Recipient.from_dict(participants_item_data)

            participants.append(participants_item)

        _time_restriction = d.pop("timeRestriction", UNSET)
        time_restriction: Union[Unset, TimeRestrictionInterval]
        if isinstance(_time_restriction, Unset):
            time_restriction = UNSET
        else:
            time_restriction = TimeRestrictionInterval.from_dict(_time_restriction)

        update_schedule_rotation_payload = cls(
            name=name,
            start_date=start_date,
            end_date=end_date,
            type=type,
            length=length,
            participants=participants,
            time_restriction=time_restriction,
        )

        update_schedule_rotation_payload.additional_properties = d
        return update_schedule_rotation_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.error_response_response_headers import ErrorResponseResponseHeaders


T = TypeVar("T", bound="ErrorResponse")


@_attrs_define
class ErrorResponse:
    """
    Attributes:
        request_id (str):
        took (float):  Default: 0.0.
        message (Union[Unset, str]):
        code (Union[Unset, int]):
        response_headers (Union[Unset, ErrorResponseResponseHeaders]):
    """

    request_id: str
    took: float = 0.0
    message: Union[Unset, str] = UNSET
    code: Union[Unset, int] = UNSET
    response_headers: Union[Unset, "ErrorResponseResponseHeaders"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        request_id = self.request_id

        took = self.took

        message = self.message

        code = self.code

        response_headers: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.response_headers, Unset):
            response_headers = self.response_headers.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "requestId": request_id,
                "took": took,
            }
        )
        if message is not UNSET:
            field_dict["message"] = message
        if code is not UNSET:
            field_dict["code"] = code
        if response_headers is not UNSET:
            field_dict["responseHeaders"] = response_headers

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.error_response_response_headers import ErrorResponseResponseHeaders

        d = src_dict.copy()
        request_id = d.pop("requestId")

        took = d.pop("took")

        message = d.pop("message", UNSET)

        code = d.pop("code", UNSET)

        _response_headers = d.pop("responseHeaders", UNSET)
        response_headers: Union[Unset, ErrorResponseResponseHeaders]
        if isinstance(_response_headers, Unset):
            response_headers = UNSET
        else:
            response_headers = ErrorResponseResponseHeaders.from_dict(_response_headers)

        error_response = cls(
            request_id=request_id,
            took=took,
            message=message,
            code=code,
            response_headers=response_headers,
        )

        error_response.additional_properties = d
        return error_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.base_incident_extra_properties import BaseIncidentExtraProperties
    from ..models.responder import Responder


T = TypeVar("T", bound="BaseIncident")


@_attrs_define
class BaseIncident:
    """
    Attributes:
        id (str):
        tiny_id (Union[Unset, str]):
        message (Union[Unset, str]):
        status (Union[Unset, str]):
        is_seen (Union[Unset, bool]):
        tags (Union[Unset, List[str]]):
        created_at (Union[Unset, datetime.datetime]):
        updated_at (Union[Unset, datetime.datetime]):
        source (Union[Unset, str]):
        owner (Union[Unset, str]):
        priority (Union[Unset, str]):
        responders (Union[Unset, List['Responder']]):
        owner_team (Union[Unset, str]):
        extra_properties (Union[Unset, BaseIncidentExtraProperties]): Map of key-value pairs to use as custom properties
            of the incident
    """

    id: str
    tiny_id: Union[Unset, str] = UNSET
    message: Union[Unset, str] = UNSET
    status: Union[Unset, str] = UNSET
    is_seen: Union[Unset, bool] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    updated_at: Union[Unset, datetime.datetime] = UNSET
    source: Union[Unset, str] = UNSET
    owner: Union[Unset, str] = UNSET
    priority: Union[Unset, str] = UNSET
    responders: Union[Unset, List["Responder"]] = UNSET
    owner_team: Union[Unset, str] = UNSET
    extra_properties: Union[Unset, "BaseIncidentExtraProperties"] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        tiny_id = self.tiny_id

        message = self.message

        status = self.status

        is_seen = self.is_seen

        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        updated_at: Union[Unset, str] = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        source = self.source

        owner = self.owner

        priority = self.priority

        responders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.responders, Unset):
            responders = []
            for responders_item_data in self.responders:
                responders_item = responders_item_data.to_dict()
                responders.append(responders_item)

        owner_team = self.owner_team

        extra_properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.extra_properties, Unset):
            extra_properties = self.extra_properties.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
            }
        )
        if tiny_id is not UNSET:
            field_dict["tinyId"] = tiny_id
        if message is not UNSET:
            field_dict["message"] = message
        if status is not UNSET:
            field_dict["status"] = status
        if is_seen is not UNSET:
            field_dict["isSeen"] = is_seen
        if tags is not UNSET:
            field_dict["tags"] = tags
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if updated_at is not UNSET:
            field_dict["updatedAt"] = updated_at
        if source is not UNSET:
            field_dict["source"] = source
        if owner is not UNSET:
            field_dict["owner"] = owner
        if priority is not UNSET:
            field_dict["priority"] = priority
        if responders is not UNSET:
            field_dict["responders"] = responders
        if owner_team is not UNSET:
            field_dict["ownerTeam"] = owner_team
        if extra_properties is not UNSET:
            field_dict["extraProperties"] = extra_properties

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.base_incident_extra_properties import BaseIncidentExtraProperties
        from ..models.responder import Responder

        d = src_dict.copy()
        id = d.pop("id")

        tiny_id = d.pop("tinyId", UNSET)

        message = d.pop("message", UNSET)

        status = d.pop("status", UNSET)

        is_seen = d.pop("isSeen", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        _created_at = d.pop("createdAt", UNSET)
        created_at: Union[Unset, datetime.datetime]
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        _updated_at = d.pop("updatedAt", UNSET)
        updated_at: Union[Unset, datetime.datetime]
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        source = d.pop("source", UNSET)

        owner = d.pop("owner", UNSET)

        priority = d.pop("priority", UNSET)

        responders = []
        _responders = d.pop("responders", UNSET)
        for responders_item_data in _responders or []:
            responders_item = Responder.from_dict(responders_item_data)

            responders.append(responders_item)

        owner_team = d.pop("ownerTeam", UNSET)

        _extra_properties = d.pop("extraProperties", UNSET)
        extra_properties: Union[Unset, BaseIncidentExtraProperties]
        if isinstance(_extra_properties, Unset):
            extra_properties = UNSET
        else:
            extra_properties = BaseIncidentExtraProperties.from_dict(_extra_properties)

        base_incident = cls(
            id=id,
            tiny_id=tiny_id,
            message=message,
            status=status,
            is_seen=is_seen,
            tags=tags,
            created_at=created_at,
            updated_at=updated_at,
            source=source,
            owner=owner,
            priority=priority,
            responders=responders,
            owner_team=owner_team,
            extra_properties=extra_properties,
        )

        base_incident.additional_properties = d
        return base_incident

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.base_incoming_feature_feature_type import BaseIncomingFeatureFeatureType
from ..models.bidirectional_callback_bidirectional_callback_type import BidirectionalCallbackBidirectionalCallbackType
from ..models.integration_type import IntegrationType
from ..models.outgoing_callback_callback_type import OutgoingCallbackCallbackType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.alert_filter import AlertFilter
    from ..models.base_incoming_feature_extra_properties import BaseIncomingFeatureExtraProperties
    from ..models.recipient import Recipient
    from ..models.team_meta import TeamMeta


T = TypeVar("T", bound="Icinga2Integration")


@_attrs_define
class Icinga2Integration:
    """
    Attributes:
        type (IntegrationType): Type of the integration. (For instance, "API" for API Integration)
        name (str): Name of the integration. Name must be unique for each integration
        id (Union[Unset, str]):
        enabled (Union[Unset, bool]): This parameter is for specifying whether the integration will be enabled or not
        owner_team (Union[Unset, TeamMeta]):
        is_global (Union[Unset, bool]):
        field_read_only (Union[Unset, List[str]]):
        suppress_notifications (Union[Unset, bool]): If enabled, notifications that come from alerts will be suppressed.
            Defaults to false
        ignore_teams_from_payload (Union[Unset, bool]): If enabled, the integration will ignore teams sent in request
            payloads. Defaults to false
        ignore_recipients_from_payload (Union[Unset, bool]): If enabled, the integration will ignore recipients sent in
            request payloads. Defaults to false
        recipients (Union[Unset, List['Recipient']]): Optional user, schedule, teams or escalation names to calculate
            which users will receive the notifications of the alert. Recipients which are exceeding the limit are ignored
        is_advanced (Union[Unset, bool]):
        ignore_responders_from_payload (Union[Unset, bool]):
        ignore_tags_from_payload (Union[Unset, bool]):
        ignore_extra_properties_from_payload (Union[Unset, bool]):
        responders (Union[Unset, List['Recipient']]):
        priority (Union[Unset, str]):
        custom_priority (Union[Unset, str]):
        tags (Union[Unset, List[str]]):
        extra_properties (Union[Unset, BaseIncomingFeatureExtraProperties]):
        assigned_team (Union[Unset, TeamMeta]):
        feature_type (Union[Unset, BaseIncomingFeatureFeatureType]):
        allow_configuration_access (Union[Unset, bool]): This parameter is for allowing or restricting the configuration
            access. If configuration access is restricted, the integration will be limited to Alert API requests and sending
            heartbeats. Defaults to false
        allow_read_access (Union[Unset, bool]):
        allow_write_access (Union[Unset, bool]): This parameter is for configuring the read-only access of integration.
            If the integration is limited to read-only access, the integration will not be authorized to perform any create,
            update or delete action within any domain. Defaults to true
        allow_delete_access (Union[Unset, bool]):
        alert_filter (Union[Unset, AlertFilter]):
        alert_actions (Union[Unset, List[str]]):
        callback_type (Union[Unset, OutgoingCallbackCallbackType]):
        send_alert_actions (Union[Unset, bool]):
        bidirectional_callback_type (Union[Unset, BidirectionalCallbackBidirectionalCallbackType]):
        send_via_marid (Union[Unset, bool]):
        send_via_oec (Union[Unset, bool]):
    """

    type: IntegrationType
    name: str
    id: Union[Unset, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    owner_team: Union[Unset, "TeamMeta"] = UNSET
    is_global: Union[Unset, bool] = UNSET
    field_read_only: Union[Unset, List[str]] = UNSET
    suppress_notifications: Union[Unset, bool] = UNSET
    ignore_teams_from_payload: Union[Unset, bool] = UNSET
    ignore_recipients_from_payload: Union[Unset, bool] = UNSET
    recipients: Union[Unset, List["Recipient"]] = UNSET
    is_advanced: Union[Unset, bool] = UNSET
    ignore_responders_from_payload: Union[Unset, bool] = UNSET
    ignore_tags_from_payload: Union[Unset, bool] = UNSET
    ignore_extra_properties_from_payload: Union[Unset, bool] = UNSET
    responders: Union[Unset, List["Recipient"]] = UNSET
    priority: Union[Unset, str] = UNSET
    custom_priority: Union[Unset, str] = UNSET
    tags: Union[Unset, List[str]] = UNSET
    extra_properties: Union[Unset, "BaseIncomingFeatureExtraProperties"] = UNSET
    assigned_team: Union[Unset, "TeamMeta"] = UNSET
    feature_type: Union[Unset, BaseIncomingFeatureFeatureType] = UNSET
    allow_configuration_access: Union[Unset, bool] = UNSET
    allow_read_access: Union[Unset, bool] = UNSET
    allow_write_access: Union[Unset, bool] = UNSET
    allow_delete_access: Union[Unset, bool] = UNSET
    alert_filter: Union[Unset, "AlertFilter"] = UNSET
    alert_actions: Union[Unset, List[str]] = UNSET
    callback_type: Union[Unset, OutgoingCallbackCallbackType] = UNSET
    send_alert_actions: Union[Unset, bool] = UNSET
    bidirectional_callback_type: Union[Unset, BidirectionalCallbackBidirectionalCallbackType] = UNSET
    send_via_marid: Union[Unset, bool] = UNSET
    send_via_oec: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        name = self.name

        id = self.id

        enabled = self.enabled

        owner_team: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner_team, Unset):
            owner_team = self.owner_team.to_dict()

        is_global = self.is_global

        field_read_only: Union[Unset, List[str]] = UNSET
        if not isinstance(self.field_read_only, Unset):
            field_read_only = self.field_read_only

        suppress_notifications = self.suppress_notifications

        ignore_teams_from_payload = self.ignore_teams_from_payload

        ignore_recipients_from_payload = self.ignore_recipients_from_payload

        recipients: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.recipients, Unset):
            recipients = []
            for recipients_item_data in self.recipients:
                recipients_item = recipients_item_data.to_dict()
                recipients.append(recipients_item)

        is_advanced = self.is_advanced

        ignore_responders_from_payload = self.ignore_responders_from_payload

        ignore_tags_from_payload = self.ignore_tags_from_payload

        ignore_extra_properties_from_payload = self.ignore_extra_properties_from_payload

        responders: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.responders, Unset):
            responders = []
            for responders_item_data in self.responders:
                responders_item = responders_item_data.to_dict()
                responders.append(responders_item)

        priority = self.priority

        custom_priority = self.custom_priority

        tags: Union[Unset, List[str]] = UNSET
        if not isinstance(self.tags, Unset):
            tags = self.tags

        extra_properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.extra_properties, Unset):
            extra_properties = self.extra_properties.to_dict()

        assigned_team: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.assigned_team, Unset):
            assigned_team = self.assigned_team.to_dict()

        feature_type: Union[Unset, str] = UNSET
        if not isinstance(self.feature_type, Unset):
            feature_type = self.feature_type.value

        allow_configuration_access = self.allow_configuration_access

        allow_read_access = self.allow_read_access

        allow_write_access = self.allow_write_access

        allow_delete_access = self.allow_delete_access

        alert_filter: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.alert_filter, Unset):
            alert_filter = self.alert_filter.to_dict()

        alert_actions: Union[Unset, List[str]] = UNSET
        if not isinstance(self.alert_actions, Unset):
            alert_actions = self.alert_actions

        callback_type: Union[Unset, str] = UNSET
        if not isinstance(self.callback_type, Unset):
            callback_type = self.callback_type.value

        send_alert_actions = self.send_alert_actions

        bidirectional_callback_type: Union[Unset, str] = UNSET
        if not isinstance(self.bidirectional_callback_type, Unset):
            bidirectional_callback_type = self.bidirectional_callback_type.value

        send_via_marid = self.send_via_marid

        send_via_oec = self.send_via_oec

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "name": name,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if owner_team is not UNSET:
            field_dict["ownerTeam"] = owner_team
        if is_global is not UNSET:
            field_dict["isGlobal"] = is_global
        if field_read_only is not UNSET:
            field_dict["_readOnly"] = field_read_only
        if suppress_notifications is not UNSET:
            field_dict["suppressNotifications"] = suppress_notifications
        if ignore_teams_from_payload is not UNSET:
            field_dict["ignoreTeamsFromPayload"] = ignore_teams_from_payload
        if ignore_recipients_from_payload is not UNSET:
            field_dict["ignoreRecipientsFromPayload"] = ignore_recipients_from_payload
        if recipients is not UNSET:
            field_dict["recipients"] = recipients
        if is_advanced is not UNSET:
            field_dict["isAdvanced"] = is_advanced
        if ignore_responders_from_payload is not UNSET:
            field_dict["ignoreRespondersFromPayload"] = ignore_responders_from_payload
        if ignore_tags_from_payload is not UNSET:
            field_dict["ignoreTagsFromPayload"] = ignore_tags_from_payload
        if ignore_extra_properties_from_payload is not UNSET:
            field_dict["ignoreExtraPropertiesFromPayload"] = ignore_extra_properties_from_payload
        if responders is not UNSET:
            field_dict["responders"] = responders
        if priority is not UNSET:
            field_dict["priority"] = priority
        if custom_priority is not UNSET:
            field_dict["customPriority"] = custom_priority
        if tags is not UNSET:
            field_dict["tags"] = tags
        if extra_properties is not UNSET:
            field_dict["extraProperties"] = extra_properties
        if assigned_team is not UNSET:
            field_dict["assignedTeam"] = assigned_team
        if feature_type is not UNSET:
            field_dict["feature-type"] = feature_type
        if allow_configuration_access is not UNSET:
            field_dict["allowConfigurationAccess"] = allow_configuration_access
        if allow_read_access is not UNSET:
            field_dict["allowReadAccess"] = allow_read_access
        if allow_write_access is not UNSET:
            field_dict["allowWriteAccess"] = allow_write_access
        if allow_delete_access is not UNSET:
            field_dict["allowDeleteAccess"] = allow_delete_access
        if alert_filter is not UNSET:
            field_dict["alertFilter"] = alert_filter
        if alert_actions is not UNSET:
            field_dict["alertActions"] = alert_actions
        if callback_type is not UNSET:
            field_dict["callback-type"] = callback_type
        if send_alert_actions is not UNSET:
            field_dict["sendAlertActions"] = send_alert_actions
        if bidirectional_callback_type is not UNSET:
            field_dict["bidirectional-callback-type"] = bidirectional_callback_type
        if send_via_marid is not UNSET:
            field_dict["sendViaMarid"] = send_via_marid
        if send_via_oec is not UNSET:
            field_dict["sendViaOEC"] = send_via_oec

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.alert_filter import AlertFilter
        from ..models.base_incoming_feature_extra_properties import BaseIncomingFeatureExtraProperties
        from ..models.recipient import Recipient
        from ..models.team_meta import TeamMeta

        d = src_dict.copy()
        type = IntegrationType(d.pop("type"))

        name = d.pop("name")

        id = d.pop("id", UNSET)

        enabled = d.pop("enabled", UNSET)

        _owner_team = d.pop("ownerTeam", UNSET)
        owner_team: Union[Unset, TeamMeta]
        if isinstance(_owner_team, Unset):
            owner_team = UNSET
        else:
            owner_team = TeamMeta.from_dict(_owner_team)

        is_global = d.pop("isGlobal", UNSET)

        field_read_only = cast(List[str], d.pop("_readOnly", UNSET))

        suppress_notifications = d.pop("suppressNotifications", UNSET)

        ignore_teams_from_payload = d.pop("ignoreTeamsFromPayload", UNSET)

        ignore_recipients_from_payload = d.pop("ignoreRecipientsFromPayload", UNSET)

        recipients = []
        _recipients = d.pop("recipients", UNSET)
        for recipients_item_data in _recipients or []:
            recipients_item = Recipient.from_dict(recipients_item_data)

            recipients.append(recipients_item)

        is_advanced = d.pop("isAdvanced", UNSET)

        ignore_responders_from_payload = d.pop("ignoreRespondersFromPayload", UNSET)

        ignore_tags_from_payload = d.pop("ignoreTagsFromPayload", UNSET)

        ignore_extra_properties_from_payload = d.pop("ignoreExtraPropertiesFromPayload", UNSET)

        responders = []
        _responders = d.pop("responders", UNSET)
        for responders_item_data in _responders or []:
            responders_item = Recipient.from_dict(responders_item_data)

            responders.append(responders_item)

        priority = d.pop("priority", UNSET)

        custom_priority = d.pop("customPriority", UNSET)

        tags = cast(List[str], d.pop("tags", UNSET))

        _extra_properties = d.pop("extraProperties", UNSET)
        extra_properties: Union[Unset, BaseIncomingFeatureExtraProperties]
        if isinstance(_extra_properties, Unset):
            extra_properties = UNSET
        else:
            extra_properties = BaseIncomingFeatureExtraProperties.from_dict(_extra_properties)

        _assigned_team = d.pop("assignedTeam", UNSET)
        assigned_team: Union[Unset, TeamMeta]
        if isinstance(_assigned_team, Unset):
            assigned_team = UNSET
        else:
            assigned_team = TeamMeta.from_dict(_assigned_team)

        _feature_type = d.pop("feature-type", UNSET)
        feature_type: Union[Unset, BaseIncomingFeatureFeatureType]
        if isinstance(_feature_type, Unset):
            feature_type = UNSET
        else:
            feature_type = BaseIncomingFeatureFeatureType(_feature_type)

        allow_configuration_access = d.pop("allowConfigurationAccess", UNSET)

        allow_read_access = d.pop("allowReadAccess", UNSET)

        allow_write_access = d.pop("allowWriteAccess", UNSET)

        allow_delete_access = d.pop("allowDeleteAccess", UNSET)

        _alert_filter = d.pop("alertFilter", UNSET)
        alert_filter: Union[Unset, AlertFilter]
        if isinstance(_alert_filter, Unset):
            alert_filter = UNSET
        else:
            alert_filter = AlertFilter.from_dict(_alert_filter)

        alert_actions = cast(List[str], d.pop("alertActions", UNSET))

        _callback_type = d.pop("callback-type", UNSET)
        callback_type: Union[Unset, OutgoingCallbackCallbackType]
        if isinstance(_callback_type, Unset):
            callback_type = UNSET
        else:
            callback_type = OutgoingCallbackCallbackType(_callback_type)

        send_alert_actions = d.pop("sendAlertActions", UNSET)

        _bidirectional_callback_type = d.pop("bidirectional-callback-type", UNSET)
        bidirectional_callback_type: Union[Unset, BidirectionalCallbackBidirectionalCallbackType]
        if isinstance(_bidirectional_callback_type, Unset):
            bidirectional_callback_type = UNSET
        else:
            bidirectional_callback_type = BidirectionalCallbackBidirectionalCallbackType(_bidirectional_callback_type)

        send_via_marid = d.pop("sendViaMarid", UNSET)

        send_via_oec = d.pop("sendViaOEC", UNSET)

        icinga_2_integration = cls(
            type=type,
            name=name,
            id=id,
            enabled=enabled,
            owner_team=owner_team,
            is_global=is_global,
            field_read_only=field_read_only,
            suppress_notifications=suppress_notifications,
            ignore_teams_from_payload=ignore_teams_from_payload,
            ignore_recipients_from_payload=ignore_recipients_from_payload,
            recipients=recipients,
            is_advanced=is_advanced,
            ignore_responders_from_payload=ignore_responders_from_payload,
            ignore_tags_from_payload=ignore_tags_from_payload,
            ignore_extra_properties_from_payload=ignore_extra_properties_from_payload,
            responders=responders,
            priority=priority,
            custom_priority=custom_priority,
            tags=tags,
            extra_properties=extra_properties,
            assigned_team=assigned_team,
            feature_type=feature_type,
            allow_configuration_access=allow_configuration_access,
            allow_read_access=allow_read_access,
            allow_write_access=allow_write_access,
            allow_delete_access=allow_delete_access,
            alert_filter=alert_filter,
            alert_actions=alert_actions,
            callback_type=callback_type,
            send_alert_actions=send_alert_actions,
            bidirectional_callback_type=bidirectional_callback_type,
            send_via_marid=send_via_marid,
            send_via_oec=send_via_oec,
        )

        icinga_2_integration.additional_properties = d
        return icinga_2_integration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.maintenance_status import MaintenanceStatus
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.maintenance_rule import MaintenanceRule
    from ..models.maintenance_time import MaintenanceTime


T = TypeVar("T", bound="Maintenance")


@_attrs_define
class Maintenance:
    """
    Attributes:
        id (Union[Unset, str]): Identifier of the maintenance meta data
        status (Union[Unset, MaintenanceStatus]): Status of the maintenance data
        rules (Union[Unset, List['MaintenanceRule']]): Rules of maintenance, which takes a list of rule objects and
            defines the maintenance rules over integrations and policies.
        time (Union[Unset, MaintenanceTime]):
        description (Union[Unset, str]): Description for maintenance data
    """

    id: Union[Unset, str] = UNSET
    status: Union[Unset, MaintenanceStatus] = UNSET
    rules: Union[Unset, List["MaintenanceRule"]] = UNSET
    time: Union[Unset, "MaintenanceTime"] = UNSET
    description: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        rules: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.rules, Unset):
            rules = []
            for rules_item_data in self.rules:
                rules_item = rules_item_data.to_dict()
                rules.append(rules_item)

        time: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.time, Unset):
            time = self.time.to_dict()

        description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if status is not UNSET:
            field_dict["status"] = status
        if rules is not UNSET:
            field_dict["rules"] = rules
        if time is not UNSET:
            field_dict["time"] = time
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.maintenance_rule import MaintenanceRule
        from ..models.maintenance_time import MaintenanceTime

        d = src_dict.copy()
        id = d.pop("id", UNSET)

        _status = d.pop("status", UNSET)
        status: Union[Unset, MaintenanceStatus]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = MaintenanceStatus(_status)

        rules = []
        _rules = d.pop("rules", UNSET)
        for rules_item_data in _rules or []:
            rules_item = MaintenanceRule.from_dict(rules_item_data)

            rules.append(rules_item)

        _time = d.pop("time", UNSET)
        time: Union[Unset, MaintenanceTime]
        if isinstance(_time, Unset):
            time = UNSET
        else:
            time = MaintenanceTime.from_dict(_time)

        description = d.pop("description", UNSET)

        maintenance = cls(
            id=id,
            status=status,
            rules=rules,
            time=time,
            description=description,
        )

        maintenance.additional_properties = d
        return maintenance

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.integration_type import IntegrationType
from ..models.outgoing_callback_new_callback_type import OutgoingCallbackNewCallbackType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.action_mapping import ActionMapping
    from ..models.alert_filter import AlertFilter
    from ..models.team_meta import TeamMeta


T = TypeVar("T", bound="AmazonSnsOutgoingIntegration")


@_attrs_define
class AmazonSnsOutgoingIntegration:
    """
    Attributes:
        type (IntegrationType): Type of the integration. (For instance, "API" for API Integration)
        name (str): Name of the integration. Name must be unique for each integration
        id (Union[Unset, str]):
        enabled (Union[Unset, bool]): This parameter is for specifying whether the integration will be enabled or not
        owner_team (Union[Unset, TeamMeta]):
        is_global (Union[Unset, bool]):
        field_read_only (Union[Unset, List[str]]):
        alert_filter (Union[Unset, AlertFilter]):
        forwarding_enabled (Union[Unset, bool]):
        forwarding_action_mappings (Union[Unset, List['ActionMapping']]):
        callback_type (Union[Unset, OutgoingCallbackNewCallbackType]):
        topic_arn (Union[Unset, str]):
        region (Union[Unset, str]):
        new_conf_type (Union[Unset, bool]):
    """

    type: IntegrationType
    name: str
    id: Union[Unset, str] = UNSET
    enabled: Union[Unset, bool] = UNSET
    owner_team: Union[Unset, "TeamMeta"] = UNSET
    is_global: Union[Unset, bool] = UNSET
    field_read_only: Union[Unset, List[str]] = UNSET
    alert_filter: Union[Unset, "AlertFilter"] = UNSET
    forwarding_enabled: Union[Unset, bool] = UNSET
    forwarding_action_mappings: Union[Unset, List["ActionMapping"]] = UNSET
    callback_type: Union[Unset, OutgoingCallbackNewCallbackType] = UNSET
    topic_arn: Union[Unset, str] = UNSET
    region: Union[Unset, str] = UNSET
    new_conf_type: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type.value

        name = self.name

        id = self.id

        enabled = self.enabled

        owner_team: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner_team, Unset):
            owner_team = self.owner_team.to_dict()

        is_global = self.is_global

        field_read_only: Union[Unset, List[str]] = UNSET
        if not isinstance(self.field_read_only, Unset):
            field_read_only = self.field_read_only

        alert_filter: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.alert_filter, Unset):
            alert_filter = self.alert_filter.to_dict()

        forwarding_enabled = self.forwarding_enabled

        forwarding_action_mappings: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.forwarding_action_mappings, Unset):
            forwarding_action_mappings = []
            for forwarding_action_mappings_item_data in self.forwarding_action_mappings:
                forwarding_action_mappings_item = forwarding_action_mappings_item_data.to_dict()
                forwarding_action_mappings.append(forwarding_action_mappings_item)

        callback_type: Union[Unset, str] = UNSET
        if not isinstance(self.callback_type, Unset):
            callback_type = self.callback_type.value

        topic_arn = self.topic_arn

        region = self.region

        new_conf_type = self.new_conf_type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type,
                "name": name,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id
        if enabled is not UNSET:
            field_dict["enabled"] = enabled
        if owner_team is not UNSET:
            field_dict["ownerTeam"] = owner_team
        if is_global is not UNSET:
            field_dict["isGlobal"] = is_global
        if field_read_only is not UNSET:
            field_dict["_readOnly"] = field_read_only
        if alert_filter is not UNSET:
            field_dict["alertFilter"] = alert_filter
        if forwarding_enabled is not UNSET:
            field_dict["forwardingEnabled"] = forwarding_enabled
        if forwarding_action_mappings is not UNSET:
            field_dict["forwardingActionMappings"] = forwarding_action_mappings
        if callback_type is not UNSET:
            field_dict["callback-type"] = callback_type
        if topic_arn is not UNSET:
            field_dict["topicArn"] = topic_arn
        if region is not UNSET:
            field_dict["region"] = region
        if new_conf_type is not UNSET:
            field_dict["newConfType"] = new_conf_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.action_mapping import ActionMapping
        from ..models.alert_filter import AlertFilter
        from ..models.team_meta import TeamMeta

        d = src_dict.copy()
        type = IntegrationType(d.pop("type"))

        name = d.pop("name")

        id = d.pop("id", UNSET)

        enabled = d.pop("enabled", UNSET)

        _owner_team = d.pop("ownerTeam", UNSET)
        owner_team: Union[Unset, TeamMeta]
        if isinstance(_owner_team, Unset):
            owner_team = UNSET
        else:
            owner_team = TeamMeta.from_dict(_owner_team)

        is_global = d.pop("isGlobal", UNSET)

        field_read_only = cast(List[str], d.pop("_readOnly", UNSET))

        _alert_filter = d.pop("alertFilter", UNSET)
        alert_filter: Union[Unset, AlertFilter]
        if isinstance(_alert_filter, Unset):
            alert_filter = UNSET
        else:
            alert_filter = AlertFilter.from_dict(_alert_filter)

        forwarding_enabled = d.pop("forwardingEnabled", UNSET)

        forwarding_action_mappings = []
        _forwarding_action_mappings = d.pop("forwardingActionMappings", UNSET)
        for forwarding_action_mappings_item_data in _forwarding_action_mappings or []:
            forwarding_action_mappings_item = ActionMapping.from_dict(forwarding_action_mappings_item_data)

            forwarding_action_mappings.append(forwarding_action_mappings_item)

        _callback_type = d.pop("callback-type", UNSET)
        callback_type: Union[Unset, OutgoingCallbackNewCallbackType]
        if isinstance(_callback_type, Unset):
            callback_type = UNSET
        else:
            callback_type = OutgoingCallbackNewCallbackType(_callback_type)

        topic_arn = d.pop("topicArn", UNSET)

        region = d.pop("region", UNSET)

        new_conf_type = d.pop("newConfType", UNSET)

        amazon_sns_outgoing_integration = cls(
            type=type,
            name=name,
            id=id,
            enabled=enabled,
            owner_team=owner_team,
            is_global=is_global,
            field_read_only=field_read_only,
            alert_filter=alert_filter,
            forwarding_enabled=forwarding_enabled,
            forwarding_action_mappings=forwarding_action_mappings,
            callback_type=callback_type,
            topic_arn=topic_arn,
            region=region,
            new_conf_type=new_conf_type,
        )

        amazon_sns_outgoing_integration.additional_properties = d
        return amazon_sns_outgoing_integration

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.maintenance_rule import MaintenanceRule
    from ..models.maintenance_time import MaintenanceTime


T = TypeVar("T", bound="CreateMaintenancePayload")


@_attrs_define
class CreateMaintenancePayload:
    """
    Attributes:
        time (MaintenanceTime):
        rules (List['MaintenanceRule']): Rules of maintenance, which takes a list of rule objects and defines the
            maintenance rules over integrations and policies.
        description (Union[Unset, str]): Description for the maintenance
    """

    time: "MaintenanceTime"
    rules: List["MaintenanceRule"]
    description: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        time = self.time.to_dict()

        rules = []
        for rules_item_data in self.rules:
            rules_item = rules_item_data.to_dict()
            rules.append(rules_item)

        description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "time": time,
                "rules": rules,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.maintenance_rule import MaintenanceRule
        from ..models.maintenance_time import MaintenanceTime

        d = src_dict.copy()
        time = MaintenanceTime.from_dict(d.pop("time"))

        rules = []
        _rules = d.pop("rules")
        for rules_item_data in _rules:
            rules_item = MaintenanceRule.from_dict(rules_item_data)

            rules.append(rules_item)

        description = d.pop("description", UNSET)

        create_maintenance_payload = cls(
            time=time,
            rules=rules,
            description=description,
        )

        create_maintenance_payload.additional_properties = d
        return create_maintenance_payload

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import webbrowser

from textual.widgets import Button


class TyButtonOpenInWebBrowser(Button):

    def __init__(self, href: str = "", **kwargs):
        super().__init__(**kwargs)
        self.href = href

    def on_button_pressed(self, event: Button.Pressed) -> None:
        webbrowser.open_new_tab(self.href)



from enum import Enum


class DeprecatedNotificationDelayAlertPolicyDelayOption(str, Enum):
    FOR_DURATION = "for-duration"
    NEXT_FRIDAY = "next-friday"
    NEXT_MONDAY = "next-monday"
    NEXT_SATURDAY = "next-saturday"
    NEXT_SUNDAY = "next-sunday"
    NEXT_THURSDAY = "next-thursday"
    NEXT_TIME = "next-time"
    NEXT_TUESDAY = "next-tuesday"
    NEXT_WEDNESDAY = "next-wednesday"
    NEXT_WEEKDAY = "next-weekday"

    def __str__(self) -> str:
        return str(self.value)



from rich.text import Text
from textual.containers import VerticalScroll
from textual.content import Content
from textual.message import Message
from textual.reactive import reactive
from textual.widget import Widget
from textual.widgets import (
    ContentSwitcher,
    DataTable,
    Label,
    Markdown,
    Pretty,
    Static,
    TabPane,
)

import tygenie.config as config
import tygenie.opsgenie as opsgenie
from tygenie.widgets.center_middle import CenterMiddle
from tygenie.widgets.tabbed_content import TygenieTabbedContent
from tygenie.widgets.tags import Tags


class AlertDetailsTabbedContent(TygenieTabbedContent):
    pass


class AlertDetailTitle(Static):

    title = reactive(Content(""))

    def watch_title(self):
        self.update(Content.from_markup("$title", title=self.title))


class AlertDetails(Widget):

    DEFAULT_CSS = """\
    RawAlertDetails {
        & #no_alert_details_label {
            height: 1fr;
            hatch: right $surface-lighten-1 70%;
        }

        & #no_alert_description_label {
            height: 1fr;
            hatch: right $surface-lighten-1 70%;
        }

        & #no_alert_tags_label {
            height: 1fr;
            hatch: right $surface-lighten-1 70%;
        }
    }
"""

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.urls = []

    def compose(self):
        self.border_title = "Alert details"
        yield AlertDetailTitle(id="alert_details_title")
        alert_details_config = config.ty_config.tygenie.get("alert_details", {})
        initial_tab = alert_details_config.get("default_tab", "details")
        with AlertDetailsTabbedContent(
            id="alert_detail_tabbed_content",
            initial=f"tabpane-{str(initial_tab).lower()}",
        ):
            with TabPane("Details", id="tabpane-details"):
                with ContentSwitcher(initial=None, id="alert_details_details_switcher"):
                    yield CenterMiddle(
                        Label("The alert doesn't have detail"),
                        id="no_alert_details_label",
                    )
                    with VerticalScroll(id="alert_detail_pretty_container"):
                        yield DataTable(id="datatable_alert_details")
            with TabPane("Tags", id="tabpane-tags"):
                with ContentSwitcher(initial=None, id="alert_details_tags_switcher"):
                    yield CenterMiddle(
                        Label("The alert doesn't have a tag"), id="no_alert_tags_label"
                    )
                    with VerticalScroll(id="alert_tags_container"):
                        yield Tags(names=[], id="alert_tags_checkboxes")
            with TabPane("Description", id="tabpane-description"):
                with ContentSwitcher(
                    initial=None, id="alert_details_description_switcher"
                ):
                    yield CenterMiddle(
                        Label("The alert doesn't have description"),
                        id="no_alert_description_label",
                    )
                    with VerticalScroll(id="alert_description_md_container"):
                        yield Markdown(None, id="pretty_alert_description")
            with TabPane("RAW", id="tabpane_raw"):
                with VerticalScroll():
                    yield Pretty(None, id="pretty_raw_alert_detail")

    class UpdateAlertDetails(Message):

        def __init__(self, alert) -> None:
            self.alert = alert.data
            super().__init__()

    async def get_alert_detail(self, opsgenie_id: str | None = None):
        if not opsgenie_id:
            return None
        alert = await opsgenie.client.api.get_alert(
            parameters={
                "identifier": opsgenie_id,
                "identifier_type": "id",
            }
        )

        if alert is not None:
            # Update alert_detail_title
            self.post_message(self.UpdateAlertDetails(alert=alert))
        else:
            self.notify(severity="error", message="Unable to get alert detail")



from textual import events
from textual.binding import Binding
from textual.message import Message
from textual.widgets import DataTable


class TygenieDataTable(DataTable, inherit_bindings=False):
    BINDINGS = [
        Binding(
            "G,pageup",
            "cursor_up_or_previous_page",
            "Go on first alert or load previous page",
            show=False,
        ),
        Binding(
            "g,pagedown",
            "cursor_down_or_next_page",
            "Go on last alert or load next page",
            show=False,
        ),
    ]

    class NextPage(Message):
        """Send a message to request next page"""

    class PreviousPage(Message):
        """Send a message to request previous page"""

    async def on_mouse_scroll_up(self, event):
        self.action_cursor_up()

    async def on_mouse_scroll_down(self, event):
        self.action_cursor_down()

    def on_key(self, event: events.Key) -> None:
        if event.key == "up":
            if self.cursor_row == 0:
                self.action_cursor_up_or_previous_page()
            else:
                self.action_cursor_up()
        elif event.key == "down":
            if self.cursor_row == self.row_count - 1:
                self.action_cursor_down_or_next_page()
            else:
                self.action_cursor_down()
        elif event.key == "left":
            self.post_message(self.PreviousPage())
        elif event.key == "right":
            self.post_message(self.NextPage())

    def action_cursor_up_or_previous_page(self):
        if self.cursor_row > 0:
            self.action_scroll_top()
        elif self.cursor_row == 0:
            self.post_message(self.PreviousPage())

    def action_cursor_down_or_next_page(self):
        if self.cursor_row < self.row_count - 1:
            self.action_scroll_bottom()
        elif self.cursor_row == self.row_count - 1:
            self.post_message(self.NextPage())



from textual.screen import Screen


class TyScreen(Screen):

    def bind(
        self,
        keys: str,
        action: str,
        *,
        description: str = "",
        show: bool = True,
        key_display: str | None = None,
    ) -> None:
        """Bind a key to an action.

        Args:
            keys: A comma separated list of keys, i.e.
            action: Action to bind to.
            description: Short description of action.
            show: Show key in UI.
            key_display: Replacement text for key, or None to use default.
        """
        self._bindings.bind(
            keys, action, description, show=show, key_display=key_display
        )
